@@ -79,13 +79,6 @@ G_DEFINE_TYPE (GsdA11yKeyboardManager, gsd_a11y_keyboard_manager, G_TYPE_OBJECT)
 
 static gpointer manager_object = NULL;
 
-#undef DEBUG_ACCESSIBILITY
-#ifdef DEBUG_ACCESSIBILITY
-#define d(str)          g_debug (str)
-#else
-#define d(str)          do { } while (0)
-#endif
-
 static void
 device_added_cb (GdkDeviceManager       *device_manager,
                  GdkDevice              *device,
@@ -112,18 +105,20 @@ set_devicepresence_handler (GsdA11yKeyboardManager *manager)
 static gboolean
 xkb_enabled (GsdA11yKeyboardManager *manager)
 {
-        gboolean have_xkb;
         int opcode, errorBase, major, minor;
 
-        have_xkb = XkbQueryExtension (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
-                                      &opcode,
-                                      &manager->priv->xkbEventBase,
-                                      &errorBase,
-                                      &major,
-                                      &minor)
-                && XkbUseExtension (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &major, &minor);
+        if (!XkbQueryExtension (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                                &opcode,
+                                &manager->priv->xkbEventBase,
+                                &errorBase,
+                                &major,
+                               &minor))
+                return FALSE;
 
-        return have_xkb;
+        if (!XkbUseExtension (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &major, &minor))
+                return FALSE;
+
+        return TRUE;
 }
 
 static XkbDescRec *
@@ -167,11 +162,10 @@ set_int (GSettings      *settings,
 
         prev_val = g_settings_get_int (settings, key);
         g_settings_set_int (settings, key, val);
-#ifdef DEBUG_ACCESSIBILITY
         if (val != prev_val) {
-                g_warning ("%s changed", key);
+                g_debug ("%s changed", key);
         }
-#endif
+
         return val != prev_val;
 }
 
@@ -185,12 +179,10 @@ set_bool (GSettings      *settings,
 
         prev_val = g_settings_get_boolean (settings, key);
         g_settings_set_boolean (settings, key, bval ? TRUE : FALSE);
-#ifdef DEBUG_ACCESSIBILITY
-                if (bval != prev_val) {
-                        d ("%s changed", key);
-                        return TRUE;
-                }
-#endif
+        if (bval != prev_val) {
+                g_debug ("%s changed", key);
+                return TRUE;
+        }
         return (bval != prev_val);
 }
 
@@ -360,7 +352,7 @@ ax_response_callback (GsdA11yKeyboardManager *manager,
         case GTK_RESPONSE_CANCEL:
 
                 /* we're reverting, so we invert sense of 'enabled' flag */
-                d ("cancelling AccessX request");
+                g_debug ("cancelling AccessX request");
                 if (revert_controls_mask == XkbStickyKeysMask) {
                         g_settings_set_boolean (settings,
                                                 "stickykeys-enable",
@@ -514,8 +506,8 @@ ax_slowkeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         GError     *error;
 
         title = enabled ?
-                _("Do you want to activate Slow Keys?") :
-                _("Do you want to deactivate Slow Keys?");
+                _("Slow Keys Turned On") :
+                _("Slow Keys Turned Off");
         message = _("You just held down the Shift key for 8 seconds.  This is the shortcut "
                     "for the Slow Keys feature, which affects the way your keyboard works.");
 
@@ -534,19 +526,20 @@ ax_slowkeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         gsd_a11y_keyboard_manager_ensure_status_icon (manager);
         manager->priv->notification = notify_notification_new (title,
                                                                message,
-                                                               "preferences-desktop-accessibility");
+                                                               "preferences-desktop-accessibility-symbolic");
         notify_notification_set_app_name (manager->priv->notification, _("Universal Access"));
         notify_notification_set_timeout (manager->priv->notification, NOTIFICATION_TIMEOUT * 1000);
+        notify_notification_set_hint (manager->priv->notification, "transient", g_variant_new_boolean (TRUE));
 
         notify_notification_add_action (manager->priv->notification,
                                         "reject",
-                                        enabled ? _("Don't activate") : _("Don't deactivate"),
+                                        enabled ? _("Turn Off") : _("Turn On"),
                                         (NotifyActionCallback) on_slow_keys_action,
                                         manager,
                                         NULL);
         notify_notification_add_action (manager->priv->notification,
                                         "accept",
-                                        enabled ? _("Activate") : _("Deactivate"),
+                                        enabled ? _("Leave On") : _("Leave Off"),
                                         (NotifyActionCallback) on_slow_keys_action,
                                         manager,
                                         NULL);
@@ -577,8 +570,8 @@ ax_slowkeys_warning_post_dialog (GsdA11yKeyboardManager *manager,
         const char *message;
 
         title = enabled ?
-                _("Do you want to activate Slow Keys?") :
-                _("Do you want to deactivate Slow Keys?");
+                _("Slow Keys Turned On") :
+                _("Slow Keys Turned Off");
         message = _("You just held down the Shift key for 8 seconds.  This is the shortcut "
                     "for the Slow Keys feature, which affects the way your keyboard works.");
 
@@ -600,16 +593,15 @@ ax_slowkeys_warning_post_dialog (GsdA11yKeyboardManager *manager,
                                GTK_STOCK_HELP,
                                GTK_RESPONSE_HELP);
         gtk_dialog_add_button (GTK_DIALOG (manager->priv->slowkeys_alert),
-                               enabled ? _("Do_n't activate") : _("Do_n't deactivate"),
+                               enabled ? _("_Turn Off") : _("_Turn On"),
                                GTK_RESPONSE_REJECT);
         gtk_dialog_add_button (GTK_DIALOG (manager->priv->slowkeys_alert),
-                               enabled ? _("_Activate") : _("_Deactivate"),
+                               enabled ? _("_Leave On") : _("_Leave Off"),
                                GTK_RESPONSE_ACCEPT);
 
-        gtk_window_set_title (GTK_WINDOW (manager->priv->slowkeys_alert),
-                              _("Slow Keys Alert"));
+        gtk_window_set_title (GTK_WINDOW (manager->priv->slowkeys_alert), "");
         gtk_window_set_icon_name (GTK_WINDOW (manager->priv->slowkeys_alert),
-                                  "input-keyboard");
+                                  "preferences-desktop-accessibility");
         gtk_dialog_set_default_response (GTK_DIALOG (manager->priv->slowkeys_alert),
                                          GTK_RESPONSE_ACCEPT);
 
@@ -647,8 +639,8 @@ ax_stickykeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         GError     *error;
 
         title = enabled ?
-                _("Do you want to activate Sticky Keys?") :
-                _("Do you want to deactivate Sticky Keys?");
+                _("Sticky Keys Turned On") :
+                _("Sticky Keys Turned Off");
         message = enabled ?
                 _("You just pressed the Shift key 5 times in a row.  This is the shortcut "
                   "for the Sticky Keys feature, which affects the way your keyboard works.") :
@@ -670,18 +662,20 @@ ax_stickykeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         gsd_a11y_keyboard_manager_ensure_status_icon (manager);
         manager->priv->notification = notify_notification_new (title,
                                                                message,
-                                                               "preferences-desktop-accessibility");
+                                                               "preferences-desktop-accessibility-symbolic");
+        notify_notification_set_app_name (manager->priv->notification, _("Universal Access"));
         notify_notification_set_timeout (manager->priv->notification, NOTIFICATION_TIMEOUT * 1000);
+        notify_notification_set_hint (manager->priv->notification, "transient", g_variant_new_boolean (TRUE));
 
         notify_notification_add_action (manager->priv->notification,
                                         "reject",
-                                        enabled ? _("Don't activate") : _("Don't deactivate"),
+                                        enabled ? _("Turn Off") : _("Turn On"),
                                         (NotifyActionCallback) on_sticky_keys_action,
                                         manager,
                                         NULL);
         notify_notification_add_action (manager->priv->notification,
                                         "accept",
-                                        enabled ? _("Activate") : _("Deactivate"),
+                                        enabled ? _("Leave On") : _("Leave Off"),
                                         (NotifyActionCallback) on_sticky_keys_action,
                                         manager,
                                         NULL);
@@ -711,8 +705,8 @@ ax_stickykeys_warning_post_dialog (GsdA11yKeyboardManager *manager,
         const char *message;
 
         title = enabled ?
-                _("Do you want to activate Sticky Keys?") :
-                _("Do you want to deactivate Sticky Keys?");
+                _("Sticky Keys Turned On") :
+                _("Sticky Keys Turned Off");
         message = enabled ?
                 _("You just pressed the Shift key 5 times in a row.  This is the shortcut "
                   "for the Sticky Keys feature, which affects the way your keyboard works.") :
@@ -737,16 +731,15 @@ ax_stickykeys_warning_post_dialog (GsdA11yKeyboardManager *manager,
                                GTK_STOCK_HELP,
                                GTK_RESPONSE_HELP);
         gtk_dialog_add_button (GTK_DIALOG (manager->priv->stickykeys_alert),
-                               enabled ? _("Do_n't activate") : _("Do_n't deactivate"),
+                               enabled ? _("_Turn Off") : _("_Turn On"),
                                GTK_RESPONSE_REJECT);
         gtk_dialog_add_button (GTK_DIALOG (manager->priv->stickykeys_alert),
-                               enabled ? _("_Activate") : _("_Deactivate"),
+                               enabled ? _("_Leave On") : _("_Leave Off"),
                                GTK_RESPONSE_ACCEPT);
 
-        gtk_window_set_title (GTK_WINDOW (manager->priv->stickykeys_alert),
-                              _("Sticky Keys Alert"));
+        gtk_window_set_title (GTK_WINDOW (manager->priv->stickykeys_alert), "");
         gtk_window_set_icon_name (GTK_WINDOW (manager->priv->stickykeys_alert),
-                                  "input-keyboard");
+                                  "preferences-desktop-accessibility");
         gtk_dialog_set_default_response (GTK_DIALOG (manager->priv->stickykeys_alert),
                                          GTK_RESPONSE_ACCEPT);
 
@@ -901,23 +894,14 @@ cb_xkb_event_filter (GdkXEvent              *xevent,
         XEvent   *xev   = (XEvent *) xevent;
         XkbEvent *xkbEv = (XkbEvent *) xevent;
 
-        /* 'event_type' is set to zero on notifying us of updates in
-         * response to client requests (including our own) and non-zero
-         * to notify us of key/mouse events causing changes (like
-         * pressing shift 5 times to enable sticky keys).
-         *
-         * We only want to update GSettings when it's in response to an
-         * explicit user input event, so require a non-zero event_type.
-         */
         if (xev->xany.type == (manager->priv->xkbEventBase + XkbEventCode) &&
-            xkbEv->any.xkb_type == XkbControlsNotify &&
-            xkbEv->ctrls.event_type != 0) {
-                d ("XKB state changed");
+            xkbEv->any.xkb_type == XkbControlsNotify) {
+                g_debug ("XKB state changed");
                 set_gsettings_from_server (manager);
         } else if (xev->xany.type == (manager->priv->xkbEventBase + XkbEventCode) &&
                    xkbEv->any.xkb_type == XkbAccessXNotify) {
                 if (xkbEv->accessx.detail == XkbAXN_AXKWarning) {
-                        d ("About to turn on an AccessX feature from the keyboard!");
+                        g_debug ("About to turn on an AccessX feature from the keyboard!");
                         /*
                          * TODO: when XkbAXN_AXKWarnings start working, we need to
                          * invoke ax_keys_warning_dialog_run here instead of in
@@ -1075,32 +1059,6 @@ gsd_a11y_keyboard_manager_stop (GsdA11yKeyboardManager *manager)
         p->stickykeys_shortcut_val = FALSE;
 }
 
-static void
-gsd_a11y_keyboard_manager_set_property (GObject        *object,
-                                        guint           prop_id,
-                                        const GValue   *value,
-                                        GParamSpec     *pspec)
-{
-        switch (prop_id) {
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
-static void
-gsd_a11y_keyboard_manager_get_property (GObject        *object,
-                                        guint           prop_id,
-                                        GValue         *value,
-                                        GParamSpec     *pspec)
-{
-        switch (prop_id) {
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
 static GObject *
 gsd_a11y_keyboard_manager_constructor (GType                  type,
                                        guint                  n_construct_properties,
@@ -1115,21 +1073,12 @@ gsd_a11y_keyboard_manager_constructor (GType                  type,
         return G_OBJECT (a11y_keyboard_manager);
 }
 
-static void
-gsd_a11y_keyboard_manager_dispose (GObject *object)
-{
-        G_OBJECT_CLASS (gsd_a11y_keyboard_manager_parent_class)->dispose (object);
-}
-
 static void
 gsd_a11y_keyboard_manager_class_init (GsdA11yKeyboardManagerClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-        object_class->get_property = gsd_a11y_keyboard_manager_get_property;
-        object_class->set_property = gsd_a11y_keyboard_manager_set_property;
         object_class->constructor = gsd_a11y_keyboard_manager_constructor;
-        object_class->dispose = gsd_a11y_keyboard_manager_dispose;
         object_class->finalize = gsd_a11y_keyboard_manager_finalize;
 
         g_type_class_add_private (klass, sizeof (GsdA11yKeyboardManagerPrivate));
