@@ -330,7 +330,7 @@ install_packages_cb (GObject *object,
         error_code = pk_results_get_error_code (results);
         if (error_code != NULL) {
                 g_warning ("failed to install file: %s, %s",
-                           pk_error_enum_to_text (pk_error_get_code (error_code)),
+                           pk_error_enum_to_string (pk_error_get_code (error_code)),
                            pk_error_get_details (error_code));
                 goto out;
         }
@@ -410,7 +410,7 @@ ignore_devices (GsdUpdatesFirmware *firmware)
         gchar *existing = NULL;
         GsdUpdatesFirmwareRequest *req;
         GPtrArray *array;
-        GString *string = NULL;
+        GString *string;
         guint i;
 
         /* get from settings */
@@ -439,8 +439,7 @@ ignore_devices (GsdUpdatesFirmware *firmware)
                                string->str);
 
         g_free (existing);
-        if (string != NULL)
-                g_string_free (string, TRUE);
+        g_string_free (string, TRUE);
 }
 
 static PkPackage *
@@ -476,7 +475,7 @@ check_available (GsdUpdatesFirmware *firmware, const gchar *filename)
         error_code = pk_results_get_error_code (results);
         if (error_code != NULL) {
                 g_warning ("failed to search file: %s, %s",
-                           pk_error_enum_to_text (pk_error_get_code (error_code)),
+                           pk_error_enum_to_string (pk_error_get_code (error_code)),
                            pk_error_get_details (error_code));
                 goto out;
         }
@@ -856,8 +855,10 @@ scan_directory (GsdUpdatesFirmware *firmware)
         /* open the directory of requests */
         dir = g_dir_open (GSD_UPDATES_FIRMWARE_MISSING_DIR, 0, &error);
         if (dir == NULL) {
-                g_warning ("failed to open directory: %s",
-                           error->message);
+                if (error->code != G_FILE_ERROR_NOENT) {
+                        g_warning ("failed to open directory: %s",
+                                   error->message);
+                }
                 g_error_free (error);
                 return;
         }
