@@ -40,7 +40,7 @@
 #define GSD_WACOM_STYLUS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_WACOM_STYLUS, GsdWacomStylusPrivate))
 
 #define WACOM_TABLET_SCHEMA "org.gnome.settings-daemon.peripherals.wacom"
-#define WACOM_DEVICE_CONFIG_BASE "/org/gnome/settings-daemon/peripherals/wacom/%s"
+#define WACOM_DEVICE_CONFIG_BASE "/org/gnome/settings-daemon/peripherals/wacom/%s/"
 #define WACOM_STYLUS_SCHEMA "org.gnome.settings-daemon.peripherals.wacom.stylus"
 #define WACOM_ERASER_SCHEMA "org.gnome.settings-daemon.peripherals.wacom.eraser"
 
@@ -54,6 +54,8 @@ struct GsdWacomStylusPrivate
 	char *name;
 	const char *icon_name;
 	GSettings *settings;
+	gboolean has_eraser;
+	int num_buttons;
 };
 
 static void     gsd_wacom_stylus_class_init  (GsdWacomStylusClass *klass);
@@ -109,6 +111,9 @@ get_icon_name_from_type (WacomStylusType type)
 {
 	switch (type) {
 	case WSTYLUS_INKING:
+	case WSTYLUS_STROKE:
+		/* The stroke pen is the same as the inking pen with
+		 * a different nib */
 		return "wacom-stylus-inking";
 	case WSTYLUS_AIRBRUSH:
 		return "wacom-stylus-airbrush";
@@ -135,6 +140,8 @@ gsd_wacom_stylus_new (GsdWacomDevice    *device,
 	stylus->priv->settings = settings;
 	stylus->priv->type = libwacom_stylus_get_type (wstylus);
 	stylus->priv->icon_name = get_icon_name_from_type (stylus->priv->type);
+	stylus->priv->has_eraser = libwacom_stylus_has_eraser (wstylus);
+	stylus->priv->num_buttons = libwacom_stylus_get_num_buttons (wstylus);
 
 	return stylus;
 }
@@ -171,6 +178,49 @@ gsd_wacom_stylus_get_device (GsdWacomStylus *stylus)
 	return stylus->priv->device;
 }
 
+gboolean
+gsd_wacom_stylus_get_has_eraser (GsdWacomStylus *stylus)
+{
+	g_return_val_if_fail (GSD_IS_WACOM_STYLUS (stylus), FALSE);
+
+	return stylus->priv->has_eraser;
+}
+
+int
+gsd_wacom_stylus_get_num_buttons (GsdWacomStylus *stylus)
+{
+	g_return_val_if_fail (GSD_IS_WACOM_STYLUS (stylus), -1);
+
+	return stylus->priv->num_buttons;
+}
+
+GsdWacomStylusType
+gsd_wacom_stylus_get_stylus_type (GsdWacomStylus *stylus)
+{
+	g_return_val_if_fail (GSD_IS_WACOM_STYLUS (stylus), WACOM_STYLUS_TYPE_UNKNOWN);
+
+	switch (stylus->priv->type) {
+	case WSTYLUS_UNKNOWN:
+		return WACOM_STYLUS_TYPE_UNKNOWN;
+	case WSTYLUS_GENERAL:
+		return WACOM_STYLUS_TYPE_GENERAL;
+	case WSTYLUS_INKING:
+		return WACOM_STYLUS_TYPE_INKING;
+	case WSTYLUS_AIRBRUSH:
+		return WACOM_STYLUS_TYPE_AIRBRUSH;
+	case WSTYLUS_CLASSIC:
+		return WACOM_STYLUS_TYPE_CLASSIC;
+	case WSTYLUS_MARKER:
+		return WACOM_STYLUS_TYPE_MARKER;
+	case WSTYLUS_STROKE:
+		return WACOM_STYLUS_TYPE_STROKE;
+	default:
+		g_assert_not_reached ();
+	}
+
+	return WACOM_STYLUS_TYPE_UNKNOWN;
+}
+
 #define GSD_WACOM_DEVICE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_WACOM_DEVICE, GsdWacomDevicePrivate))
 
 /* we support two types of settings:
@@ -374,7 +424,7 @@ add_stylus_to_device (GsdWacomDevice *device,
 		    libwacom_stylus_is_eraser (wstylus) == FALSE)
 			return;
 
-		stylus_settings_path = g_strdup_printf ("%s/0x%x", settings_path, id);
+		stylus_settings_path = g_strdup_printf ("%s0x%x/", settings_path, id);
 		if (device->priv->type == WACOM_TYPE_STYLUS) {
 			settings = g_settings_new_with_path (WACOM_STYLUS_SCHEMA, stylus_settings_path);
 			stylus = gsd_wacom_stylus_new (device, wstylus, settings);
@@ -644,6 +694,23 @@ gsd_wacom_device_list_styli (GsdWacomDevice *device)
 	return g_list_copy (device->priv->styli);
 }
 
+GsdWacomStylus *
+gsd_wacom_device_get_stylus_for_type (GsdWacomDevice     *device,
+				      GsdWacomStylusType  type)
+{
+	GList *l;
+
+	g_return_val_if_fail (GSD_IS_WACOM_DEVICE (device), NULL);
+
+	for (l = device->priv->styli; l != NULL; l = l->next) {
+		GsdWacomStylus *stylus = l->data;
+
+		if (gsd_wacom_stylus_get_stylus_type (stylus) == type)
+			return stylus;
+	}
+	return NULL;
+}
+
 const char *
 gsd_wacom_device_get_name (GsdWacomDevice *device)
 {
@@ -734,6 +801,46 @@ gsd_wacom_device_get_device_type (GsdWacomDevice *device)
 	return device->priv->type;
 }
 
+gint *
+gsd_wacom_device_get_area (GsdWacomDevice *device)
+{
+	int i, id;
+	XDevice *xdevice;
+	Atom area, realtype;
+	int rc, realformat;
+	unsigned long nitems, bytes_after;
+	unsigned char *data = NULL;
+	gint *device_area;
+
+	g_object_get (device->priv->gdk_device, "device-id", &id, NULL);
+
+	area = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "Wacom Tablet Area", False);
+
+	gdk_error_trap_push ();
+	xdevice = XOpenDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), id);
+	if (gdk_error_trap_pop () || (device == NULL))
+		return NULL;
+
+	gdk_error_trap_push ();
+	rc = XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+				 xdevice, area, 0, 4, False,
+				 XA_INTEGER, &realtype, &realformat, &nitems,
+				 &bytes_after, &data);
+	if (gdk_error_trap_pop () || rc != Success || realtype == None || bytes_after != 0 || nitems != 4) {
+		XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
+		return NULL;
+	}
+
+	device_area = g_new0 (int, nitems);
+	for (i = 0; i < nitems; i++)
+		device_area[i] = ((long*)data)[i];
+
+	XFree (data);
+	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdevice);
+
+	return device_area;
+}
+
 const char *
 gsd_wacom_device_type_to_string (GsdWacomDeviceType type)
 {
@@ -851,3 +958,32 @@ gsd_wacom_device_create_fake_x201 (void)
 
 	return devices;
 }
+
+GList *
+gsd_wacom_device_create_fake_intuos4 (void)
+{
+	GsdWacomDevice *device;
+	GList *devices;
+
+	device = gsd_wacom_device_create_fake (WACOM_TYPE_STYLUS,
+					       "Intuos 4 M 6x9",
+					       "Wacom Intuos4 6x9 stylus");
+	devices = g_list_prepend (NULL, device);
+
+	device = gsd_wacom_device_create_fake (WACOM_TYPE_ERASER,
+					       "Intuos 4 M 6x9",
+					       "Wacom Intuos4 6x9 eraser");
+	devices = g_list_prepend (devices, device);
+
+	device = gsd_wacom_device_create_fake (WACOM_TYPE_PAD,
+					       "Intuos 4 M 6x9",
+					       "Wacom Intuos4 6x9 pad");
+	devices = g_list_prepend (devices, device);
+
+	device = gsd_wacom_device_create_fake (WACOM_TYPE_CURSOR,
+					       "Intuos 4 M 6x9",
+					       "Wacom Intuos4 6x9 cursor");
+	devices = g_list_prepend (devices, device);
+
+	return devices;
+}
